; A/D Toolkit V 1.0   *************************************************

; Add-on procedures:  MOVE_B s,d,x     move x bytes from s to d
;                     MOVE_W s,d,x     move x words from s to d
;                     MOVE_L s,d,x     move x longwords from s to d
;                     RETCH adr        return space reserved at adr
;                     COMPR x,a,f      compress s data at a with fact f
;                     READ_AD x,y,a,t  read x A/D values, each y times,
;                                      store at a, trigger by t 
;                     READ_SD c,x,y,a,t read x S/D values of channel c,
;                                      each y times, store at a, trig t
;                     SEND_SD c,x,y,a  write x D/S values to channel c,
;                                      the values are stored at a
;                     SMUL_SD c1,c2,x,y,a1,a2  write x D/S words stored
;                                      at a1,a2 alternating to c1,c2
;                     CLKP_SD c,x,y    clock(+) c, x times, timebase y
;                     CLKR_SD c,x,r,y  clock(+) c, x + r ramped pulses
;                     SET_SD c,v       switch D/S channel c off (v=0)
;                                      or on (v=128)
; Add-on functions:   adr=ALLCH(l)     Allocates l Bytes in CH at adr
;                     val=FREQ(x,a)    counts T of x data stored at a
;                     val=AD_VOLT      reads A/D
;                     val=SD_VOLT(c)   reads S/D channel c 


; For hardware arrangements, note the following, if full compatibility
; with actual and future JFC A/D-hard-/software should remain

; Reserved hardware address range:     $FC00 ... $FFFF 
; actually used by:

; A/D   $FC00                          A/D_1
; D/A   $FD00                          reserved     
; S/D   $FE00 ... 03                   A/D_2 in
; D/S   $FF00 ... 03 / $FF80 ... 03    A/D_2 out


CONSTANTS ;************************************************************

BP_INIT         EQU     $110
BV_RIP          EQU     $58
CA_GTLIN        EQU     $118

SOFTWARE_INST ;********************************************************

        LEA     NAM_TAB,A1      ;define add-ons
        MOVE    BP_INIT,A2
        JMP     (A2)            ;initialize and return to basic

ROM_INST ;*************************************************************

        DC.L    $4AFB0001       ;ROM recognition
        DC.W    NAM_TAB-ROM_INST ;name table
        DC.W    0               ;no inst routine
        DC.W    38
        DC.B    " JFC   QL_A/D Toolkit  V 1.0    1987",10

NAM_TAB ;**************************************************************

        DC.W    12              ;12 proc
        DC.W    RETCH-*         ;begin nr. 1 at RETCH
        DC.B    5,"RETCH"       ;length and name
        DC.W    MOVE_B-*   
        DC.B    6,"MOVE_B "
        DC.W    MOVE_W-*  
        DC.B    6,"MOVE_W "
        DC.W    MOVE_L-*   
        DC.B    6,"MOVE_L "
        DC.W    COMPR-*    
        DC.B    5,"COMPR"
        DC.W    READ_AD-*
        DC.B    7,"READ_AD"
        DC.W    READ_SD-*
        DC.B    7,"READ_SD"
        DC.W    SEND_SD-*
        DC.B    7,"SEND_SD"
        DC.W    SMUL_SD-*
        DC.B    7,"SMUL_SD"
        DC.W    CLKP_SD-*
        DC.B    7,"CLKP_SD"
        DC.W    CLKR_SD-*
        DC.B    7,"CLKR_SD"
        DC.W    SET_SD-* 
        DC.B    6,"SET_SD"
        DC.W    0               ;end proc
        DC.W    4               ;4 func
        DC.W    ALLCH-*         ;begin nr. 1 at ALLCH
        DC.B    5,"ALLCH"       ;length and name
        DC.W    FREQ-*   
        DC.B    4,"FREQ "
        DC.W    AD_VOLT-*
        DC.B    7,"AD_VOLT"
        DC.W    SD_VOLT-*
        DC.B    7,"SD_VOLT"
        DC.W    0               ;end func
        DC.W    0               ;end def

ERRORS ;***************************************************************

BAD_PARA
        MOVEQ   #-15,D0         ;error bad parameter
ERR
        RTS

SUBS ******************************************************************
               
GT_LIN
        MOVE.W  CA_GTLIN,A2
        JSR     (A2)
        BNE     GTL_RET
        CMP.B   D5,D3
GTL_RET
        RTS

CREATE_FP
        MOVEA.L BV_RIP(A6),A1
        SUBQ    #6,A1
        CLR.W   0(A6,A1.L)
        TST.L   D1
        BEQ.S   CR_FP2
        MOVE.W  #$820,D2
CR_FP1
        SUBQ.W  #1,D2
        ASL.L   #1,D1
        BVC.S   CR_FP1
        ROXR.L  #1,D1
        MOVE.W  D2,0(A6,A1.L)
CR_FP2
        MOVE.L  D1,2(A6,A1.L)
        MOVE.L  A1,BV_RIP(A6)
        MOVEQ   #2,D4                               
        MOVEQ   #0,D0
        RTS

PROCEDURES ************************************************************

RETCH
        MOVEQ   #1,D5           ;one parameter
        BSR     GT_LIN          ;get argument
        BNE     BAD_PARA
        MOVEA.L 0(A6,A1.L),A0
        MOVEQ   #$19,D0          
        TRAP    #1              ;return memory
        RTS

MOVE_B
        MOVEQ   #3,D5           ;three parameters
        BSR     GT_LIN
        BNE     BAD_PARA
        MOVEA.L 0(A6,A1.L),A3   ;source address
        MOVEA.L 4(A6,A1.L),A4   ;destination address
        MOVE.L  8(A6,A1.L),D1   ;number of bytes
        SUBQ    #1,D1
M_B1
        MOVE.B  (A3)+,(A4)+     ;move bytes
        DBF     D1,M_B1
        MOVEQ   #0,D0
        RTS
        
MOVE_W
        MOVEQ   #3,D5           ;three parameters
        BSR     GT_LIN
        BNE     BAD_PARA
        MOVEA.L 0(A6,A1.L),A3   ;source address
        MOVEA.L 4(A6,A1.L),A4   ;destination address
        MOVE.L  8(A6,A1.L),D1   ;number of words
        SUBQ    #1,D1
M_W1
        MOVE.W  (A3)+,(A4)+     ;move words
        DBF     D1,M_W1
        MOVEQ   #0,D0
        RTS
        
MOVE_L
        MOVEQ   #3,D5           ;three parameters
        BSR     GT_LIN
        BNE     BAD_PARA
        MOVEA.L 0(A6,A1.L),A3   ;source address
        MOVEA.L 4(A6,A1.L),A4   ;destination address
        MOVE.L  8(A6,A1.L),D1   ;number of longwords
        SUBQ    #1,D1
M_L1
        MOVE.L  (A3)+,(A4)+     ;move longwords
        DBF     D1,M_L1
        MOVEQ   #0,D0
        RTS

COMPR
        MOVEQ   #3,D5           ;three arguments
        BSR     GT_LIN          ;get arguments
        BNE     BAD_PARA
        MOVE.L  0(A6,A1.L),D0   ;number of vals
        SUBQ    #1,D0
        MOVE.L  4(A6,A1.L),A4   ;address of vals
        MOVE.L  A4,A2
        MOVE.L  8(A6,A1.L),D1   ;compression factor
        MOVEQ.L #0,D4
NEXT_VAL
        CLR.L   D5
        MOVE.L  D1,D2
        SUBQ    #1,D2
SUM
        MOVE.B  (A4)+,D4
        ADD.W   D4,D5
        DBF     D2,SUM
        DIVU    D1,D5
        MOVE.B  D5,(A2)+
        DBF     D0,NEXT_VAL
        MOVEQ   #0,D0
        RTS                        

READ_AD
        MOVEQ   #4,D5           ;four arguments
        BSR     GT_LIN
        BNE     BAD_PARA
        MOVE.L  0(A6,A1.L),D1   ;number of bytes
        SUBQ    #1,D1
        MOVE.L  4(A6,A1.L),D2   ;number of times each byte
        MOVEA.L #$FC00,A4       ;A/D-adress
        MOVEA.L 8(A6,A1.L),A5   ;storeadress
        MOVE.L  $C(A6,A1.L),D3  ;trigger value
READ_BEG
        TRAP    #0
        ORI     #$700,SR
        MOVEQ   #4,D4           ;clear A/D reg
        LEA     READ_QUICK,A2   ;quick reading address
        CMPI    #1,D2           ;quick reading ?
        BEQ     READ_TRIG       ;quick reading !
        LEA     READ_MORE,A2    ;more reading !
READ_TRIG
        CMP.L   #0,D3           ;no trigger ?
        BNE     TRIG            ;trigger !
        JMP     (A2)
TRIG
        MOVEA.L #$18000,A3      ;clockaddress
        MOVE.L  (A3),D7         ;starttime
        ADDQ.L  #8,D7           ;timeout 8 sec
        MOVE    D2,D5           ;loop value
        SUBQ    #1,D5
        MULU    #$A,D5
TRIG_DIR        
        CMPI    #0,D3           ;trigger direction
        BMI     TRIG_DOWN
TRIG_UP
        CMP.L   (A3),D7
        BMI     TRIGGED         ;timeout
        MOVE.W  D5,D6           ;loop value
        MOVE.B  (A4),D4         ;read A/D
        CMP.W   D3,D4           ;test
        BPL     TRIG_UP
TRIG_UP1
        DBF     D6,TRIG_UP1        
        MOVE.B  (A4),D4         ;read A/D
        CMP.W   D3,D4           ;test
        BMI     TRIG_UP
TRIGGED
        JMP     (A2)            ;read now
TRIG_DOWN
        NEG     D3
TRIG_DN        
        CMP.L   (A3),D7
        BMI     TRIGGED         ;timeout
        MOVE.L  D5,D6           ;loop value
        MOVE.B  (A4),D4         ;read A/D
        CMP.W   D3,D4           ;test
        BMI     TRIG_DN
TRIG_DN1
        DBF     D6,TRIG_DN1
        MOVE.B  (A4),D4         ;read A/D
        CMP.W   D3,D4           ;test
        BPL     TRIG_DN
        JMP     (A2)            ;read now
READ_QUICK        
        MOVE.B  (A4),(A5)+      ;read A/D
        DBF     D1,READ_QUICK   ;D1 times
        ANDI    #$D8FF,SR
        RTS
READ_MORE
        MOVE.L  D2,D3
        SUBQ    #1,D3
        MOVEQ   #0,D5           ;clear sum reg  
READ_M1
        MOVE.B  (A4),D4         ;read A/D in D4
        ADD.W   D4,D5           ;add in D5
        DBF     D3,READ_M1      ;D3 times
        DIVU    D2,D5
        MOVE.B  D5,(A5)+        ;store medium value
        DBF     D1,READ_MORE
        ANDI    #$D8FF,SR
        RTS

READ_SD
        MOVEQ   #5,D5           ;five arguments
        BSR     GT_LIN
        BNE     BAD_PARA
        MOVE.L  0(A6,A1.L),A4   ;load channel offset address
        ADDA.L  #$FE00,A4       ;add S/D base address
        MOVE.L  4(A6,A1.L),D1   ;number of bytes
        SUBQ    #1,D1
        MOVE.L  8(A6,A1.L),D2   ;number of times each byte
        MOVEA.L $C(A6,A1.L),A5  ;store address
        MOVE.L  $10(A6,A1.L),D3 ;trigger value
        MOVE.L  D1,-(A7)        ;number on stack
        MOVE.L  A5,-(A7)        ;store address on stack
        BSR     READ_BEG
        MOVEA.L (A7)+,A5        ;restore address
        MOVE.L  (A7)+,D1        ;restore number of bytes
READ_SD_CONVERT
        BTST    #0,(A5)         ;switch on ?
        BEQ     R_SD_OFF        ;switch off !
        MOVE.B  #$80,(A5)+      ;on=128
        BRA     R_SD_CONV
R_SD_OFF
        MOVE.B  #0,(A5)+        ;off=0
R_SD_CONV
        DBF     D1,READ_SD_CONVERT
        MOVEQ   #0,D0
        RTS        

SEND_SD
        MOVEQ   #4,D5           ;four parameters
        BSR     GT_LIN          ;get arguments
        BNE     BAD_PARA
        TRAP    #0
        ORI     #$700,SR
        MOVE.L  0(A6,A1.L),A4   ;load channel offset address
        ADDA.L  #$FF00,A4       ;add S/D base address
        MOVE.L  4(A6,A1.L),D1   ;number of vals
        SUBQ    #1,D0
        MOVE.L  8(A6,A1.L),D2   ;number of times each byte
        MOVEA.L $C(A6,A1.L),A5  ;source address
        MOVEQ   #0,D4           ;clear on/off offset reg
        CMP.L   #1,D2           ;quick writing ?
        BNE     SEND_MORE
SEND_QUICK
        MOVE.B  (A5)+,D4        ;switch condition
        MOVE.B  0(A4,D4),D0     ;write D/S
        DBF     D1,SEND_QUICK   ;D1 times
        BRA     SEND_END
SEND_MORE
        MOVE.L  D2,D3
        SUBQ    #1,D3
        MOVEQ   #0,D4           ;dummy move
SEND_M1
        MOVE.B  (A5),D4         ;switch condition
        ADD.W   D4,D5           ;dummy add
        DBF     D3,SEND_M1
        DIVU    D2,D5           ;dummy div
        ADDQ.L  #1,A5
        MOVE.B  0(A4,D4),D0     ;write D/S
        DBF     D1,SEND_MORE
SEND_END
        ANDI    #$D8FF,SR
        MOVEQ   #0,D0
        RTS

SMUL_SD
        MOVEQ   #6,D5           ;six parameters
        BSR     GT_LIN          ;get arguments
        BNE     BAD_PARA
        TRAP    #0
        ORI     #$700,SR
        MOVE.L  0(A6,A1.L),A4   ;load first channel offset address
        ADDA.L  #$FF00,A4       ;add S/D base address
        MOVE.L  4(A6,A1.L),A5   ;load second channel offset address
        ADDA.L  #$FF00,A5       ;add S/D base address
        MOVE.L  8(A6,A1.L),D1   ;number of vals
        SUBQ    #1,D0
        MOVE.L  $C(A6,A1.L),D2  ;number of times each byte
        MOVEA.L $10(A6,A1.L),A2 ;source address first channel
        MOVEA.L $14(A6,A1.L),A3 ;source address second channel
        MOVEQ   #0,D4           ;clear on/off offset reg
        CMP.L   #1,D2           ;quick writing ?
        BNE     SMUL_MORE
SMUL_QUICK
        MOVE.B  (A2)+,D4        ;first switch condition
        MOVE.B  (A3)+,D5        ;second switch condition
        MOVE.B  0(A4,D4),D0     ;set first D/S
        MOVE.B  0(A5,D5),D0     ;set second D/S
        DBF     D1,SMUL_QUICK   ;D1 times
        BRA     SMUL_END
SMUL_MORE
        MOVE.L  D2,D3
        SUBQ    #1,D3
        MOVE.B  (A2)+,D4        ;first switch and next condition
SMUL_M1
        MOVE.B  (A3),D5         ;second switch condition
        ADD.W   D5,D6           ;dummy add
        DBF     D3,SMUL_M1
        DIVU    D2,D6           ;dummy div
        ADDQ.L  #1,A3           ;next condition
        MOVE.B  0(A4,D4),D0     ;set first D/S
        MOVE.B  0(A5,D5),D0     ;set second D/S
        DBF     D1,SMUL_MORE
SMUL_END
        ANDI    #$D8FF,SR
        MOVEQ   #0,D0
        RTS

CLKP_SD
        MOVEQ   #3,D5           ;three arguments
        BSR     GT_LIN
        BNE     BAD_PARA
        TRAP    #0
        ORI     #$700,SR
        MOVE.L  0(A6,A1.L),A4   ;channel offset address
        ADDA.L  #$FF00,A4       ;S/D base address
        MOVE.L  4(A6,A1.L),D0   ;number of clock pulses
        SUBQ    #1,D0
        MOVE.L  8(A6,A1.L),D2   ;time delay
        SUBQ    #1,D2
CLK
        MOVE.B  $80(A4),D4      ;clock pulse        
        MOVE.B  (A4),D4         ;clock low
        MOVE.L  D2,D3
CLK_WAIT
        DBF     D3,CLK_WAIT
        DBF     D0,CLK
        ANDI    #$D8FF,SR
        MOVEQ   #0,D0
        RTS

CLKR_SD
        MOVEQ   #4,D5           ;four arguments
        BSR     GT_LIN
        BNE     BAD_PARA
        TRAP    #0
        ORI     #$700,SR
        MOVE.L  0(A6,A1.L),A4   ;channel offset address
        ADDA.L  #$FF00,A4       ;S/D base address
        MOVE.L  4(A6,A1.L),D1   ;number of ramp pulses
        SUBQ    #1,D1
        MOVE.L  8(A6,A1.L),D0   ;number of clock pulses
        SUBQ    #1,D0
        MOVE.L  $C(A6,A1.L),D2  ;time delay
        MOVE.L  D1,D3           ;ramp counter
CLKR_UP
        MOVE.B  $80(A4),D4      ;clock pulse        
        MOVE.B  (A4),D4         ;clock low
        MOVE.L  D2,D6           ;short delay
        ADD.L   D3,D6           ;add falling ramp delay
CLKR_UP_WAIT
        DBF     D6,CLKR_UP_WAIT
        DBF     D3,CLKR_UP
CLKR
        MOVE.B  $80(A4),D4      ;clock pulse
        MOVE.B  (A4),D4         ;clock low
        MOVE.L  D2,D6           ;short delay
        ADD.L   D3,D7           ;dummy add
CLKR_WAIT
        DBF     D6,CLKR_WAIT
        DBF     D0,CLKR
CLKR_DOWN
        MOVE.B  $80(A4),D4      ;clock pulse        
        MOVE.B  (A4),D4         ;clock low
        MOVE.L  D2,D6           ;short delay
        ADD.L   #1,D2           ;add rising ramp delay
CLKR_DOWN_WAIT
        DBF     D6,CLKR_DOWN_WAIT
        DBF     D1,CLKR_DOWN
        ANDI    #$D8FF,SR
        MOVEQ   #0,D0
        RTS

SET_SD
        MOVEQ   #2,D5           ;two arguments
        BSR     GT_LIN
        BNE     BAD_PARA
        MOVE.L  0(A6,A1.L),A4   ;load channel offset address
        ADDA.L  #$FF00,A4       ;add S/D base address
        MOVE.L  4(A6,A1.L),D1   ;switch condition
        BEQ     SET_OFF
        MOVE.L  #$80,D1
        MOVE.B  0(A4,D1),D2     ;set switch on
        MOVEQ   #0,D0
        RTS
SET_OFF
        MOVE.B  (A4),D2         ;set switch off
        MOVEQ   #0,D0
        RTS

FUNCTIONS ;************************************************************

ALLCH
        MOVEQ   #1,D5           ;one parameter
        BSR     GT_LIN          ;get argument
        BNE     BAD_PARA
        MOVE.L  0(A6,A1.L),D1   ;number of bytes
        MOVEQ   #-1,D2          ;this job
        MOVEQ   #$18,D0
        TRAP    #1
        TST.B   D0
        BNE     ERR
        MOVE.L  A0,D1
        BRA     CREATE_FP                

FREQ
        MOVEQ   #2,D5           ;two arguments
        BSR     GT_LIN          ;get argument
        BNE     BAD_PARA
        MOVE.L  A5,A2
        MOVE.L  0(A6,A1.L),D4   ;number of bytes
        MOVE.L  D4,D5           ;store number of bytes
        MOVEA.L 4(A6,A1.L),A4   ;address of first byte
        MOVEA.L A4,A5           ;store address
        CLR.L   D1              ;clear return reg
        CLR.L   D2
        CLR.L   D3
        CLR.L   D7              ;clear max reg
        CLR.L   D6              ;clear actual reg
        MOVE.B  (A4)+,D7        ;first max
F_MAX
        MOVE.B  (A4)+,D6        ;actual reg
        CMP.W   D6,D7
        BPL     F_NEXT          ;actual lower !
        MOVE.B  D6,D7           ;store new max
F_NEXT
        DBF     D4,F_MAX
        SUBQ    #8,D7           ;upper max tolerance
        MOVE.W  D7,D6           ;lower max
        SUBQ    #5,D6           ;lower max tolearance
        MOVE.W  D5,D4           ;reload number
        MOVEA.L A5,A4           ;reload address
        CLR.L   D5
        BSR     F_FIND_MIN
        BSR     F_FIND_MAX
        MOVE.W  D4,D1           ;first max
        BSR     F_FIND_MIN
        BSR     F_FIND_MAX
        MOVE.W  D4,D2           ;second max
        SUB.L   D2,D1
        MOVE.L  A2,A5
        BRA     CREATE_FP        
F_FIND_MIN
        MOVE.B  (A4)+,D5
        CMP.W   D5,D6
        DBPL    D4,F_FIND_MIN
        RTS
F_FIND_MAX
        MOVE.B  (A4)+,D5
        CMP.W   D5,D7
        DBMI    D4,F_FIND_MAX
        RTS                

AD_VOLT
        MOVEA.L #$FC00,A4       ;A/D base
        CLR.L   D1              ;clear return reg
        CLR.L   D2              ;clear A/D sum reg
        CLR.L   D4              ;clear A/D reg
        MOVE.B  (A4),D2         ;read A/D in D2
        MOVEQ   #6,D3
AD_V
        MOVE.B  (A4),D4         ;read A/D in D4
        ADD.W   D4,D2           ;add in D2
        DBF     D3,AD_V
        DIVU    #8,D2           ;mean value
        MOVE.B  D2,D1           ;cut off division rest
        BRA     CREATE_FP

SD_VOLT
        MOVEQ   #1,D5           ;one parameter
        BSR     GT_LIN
        BNE     BAD_PARA
        MOVE.L  0(A6,A1.L),A4   ;channel offset address
        ADDA.L  #$FE00,A4       ;add S/D base address
        CLR.L   D1
        MOVE.B  (A4),D1
        BTST    #0,D1           ;switch on ?
        BEQ     SD_OFF          ;switch off
        MOVE.B  #$80,D1         ;on=128
        BRA     CREATE_FP       ;return 128
SD_OFF
        CLR.L   D1              ;off=0
        BRA     CREATE_FP       ;return 0
